// Code generated by ent, DO NOT EDIT.

package ent

import (
	"fmt"
	"strings"

	"entgo.io/ent"
	"entgo.io/ent/dialect/sql"
	"github.com/google/uuid"
	"github.com/in-toto/archivista/ent/attestationcollection"
	"github.com/in-toto/archivista/ent/statement"
)

// AttestationCollection is the model entity for the AttestationCollection schema.
type AttestationCollection struct {
	config `json:"-"`
	// ID of the ent.
	ID uuid.UUID `json:"id,omitempty"`
	// Name holds the value of the "name" field.
	Name string `json:"name,omitempty"`
	// Edges holds the relations/edges for other nodes in the graph.
	// The values are being populated by the AttestationCollectionQuery when eager-loading is set.
	Edges                             AttestationCollectionEdges `json:"edges"`
	statement_attestation_collections *uuid.UUID
	selectValues                      sql.SelectValues
}

// AttestationCollectionEdges holds the relations/edges for other nodes in the graph.
type AttestationCollectionEdges struct {
	// Attestations holds the value of the attestations edge.
	Attestations []*Attestation `json:"attestations,omitempty"`
	// Statement holds the value of the statement edge.
	Statement *Statement `json:"statement,omitempty"`
	// loadedTypes holds the information for reporting if a
	// type was loaded (or requested) in eager-loading or not.
	loadedTypes [2]bool
	// totalCount holds the count of the edges above.
	totalCount [2]map[string]int

	namedAttestations map[string][]*Attestation
}

// AttestationsOrErr returns the Attestations value or an error if the edge
// was not loaded in eager-loading.
func (e AttestationCollectionEdges) AttestationsOrErr() ([]*Attestation, error) {
	if e.loadedTypes[0] {
		return e.Attestations, nil
	}
	return nil, &NotLoadedError{edge: "attestations"}
}

// StatementOrErr returns the Statement value or an error if the edge
// was not loaded in eager-loading, or loaded but was not found.
func (e AttestationCollectionEdges) StatementOrErr() (*Statement, error) {
	if e.Statement != nil {
		return e.Statement, nil
	} else if e.loadedTypes[1] {
		return nil, &NotFoundError{label: statement.Label}
	}
	return nil, &NotLoadedError{edge: "statement"}
}

// scanValues returns the types for scanning values from sql.Rows.
func (*AttestationCollection) scanValues(columns []string) ([]any, error) {
	values := make([]any, len(columns))
	for i := range columns {
		switch columns[i] {
		case attestationcollection.FieldName:
			values[i] = new(sql.NullString)
		case attestationcollection.FieldID:
			values[i] = new(uuid.UUID)
		case attestationcollection.ForeignKeys[0]: // statement_attestation_collections
			values[i] = &sql.NullScanner{S: new(uuid.UUID)}
		default:
			values[i] = new(sql.UnknownType)
		}
	}
	return values, nil
}

// assignValues assigns the values that were returned from sql.Rows (after scanning)
// to the AttestationCollection fields.
func (ac *AttestationCollection) assignValues(columns []string, values []any) error {
	if m, n := len(values), len(columns); m < n {
		return fmt.Errorf("mismatch number of scan values: %d != %d", m, n)
	}
	for i := range columns {
		switch columns[i] {
		case attestationcollection.FieldID:
			if value, ok := values[i].(*uuid.UUID); !ok {
				return fmt.Errorf("unexpected type %T for field id", values[i])
			} else if value != nil {
				ac.ID = *value
			}
		case attestationcollection.FieldName:
			if value, ok := values[i].(*sql.NullString); !ok {
				return fmt.Errorf("unexpected type %T for field name", values[i])
			} else if value.Valid {
				ac.Name = value.String
			}
		case attestationcollection.ForeignKeys[0]:
			if value, ok := values[i].(*sql.NullScanner); !ok {
				return fmt.Errorf("unexpected type %T for field statement_attestation_collections", values[i])
			} else if value.Valid {
				ac.statement_attestation_collections = new(uuid.UUID)
				*ac.statement_attestation_collections = *value.S.(*uuid.UUID)
			}
		default:
			ac.selectValues.Set(columns[i], values[i])
		}
	}
	return nil
}

// Value returns the ent.Value that was dynamically selected and assigned to the AttestationCollection.
// This includes values selected through modifiers, order, etc.
func (ac *AttestationCollection) Value(name string) (ent.Value, error) {
	return ac.selectValues.Get(name)
}

// QueryAttestations queries the "attestations" edge of the AttestationCollection entity.
func (ac *AttestationCollection) QueryAttestations() *AttestationQuery {
	return NewAttestationCollectionClient(ac.config).QueryAttestations(ac)
}

// QueryStatement queries the "statement" edge of the AttestationCollection entity.
func (ac *AttestationCollection) QueryStatement() *StatementQuery {
	return NewAttestationCollectionClient(ac.config).QueryStatement(ac)
}

// Update returns a builder for updating this AttestationCollection.
// Note that you need to call AttestationCollection.Unwrap() before calling this method if this AttestationCollection
// was returned from a transaction, and the transaction was committed or rolled back.
func (ac *AttestationCollection) Update() *AttestationCollectionUpdateOne {
	return NewAttestationCollectionClient(ac.config).UpdateOne(ac)
}

// Unwrap unwraps the AttestationCollection entity that was returned from a transaction after it was closed,
// so that all future queries will be executed through the driver which created the transaction.
func (ac *AttestationCollection) Unwrap() *AttestationCollection {
	_tx, ok := ac.config.driver.(*txDriver)
	if !ok {
		panic("ent: AttestationCollection is not a transactional entity")
	}
	ac.config.driver = _tx.drv
	return ac
}

// String implements the fmt.Stringer.
func (ac *AttestationCollection) String() string {
	var builder strings.Builder
	builder.WriteString("AttestationCollection(")
	builder.WriteString(fmt.Sprintf("id=%v, ", ac.ID))
	builder.WriteString("name=")
	builder.WriteString(ac.Name)
	builder.WriteByte(')')
	return builder.String()
}

// NamedAttestations returns the Attestations named value or an error if the edge was not
// loaded in eager-loading with this name.
func (ac *AttestationCollection) NamedAttestations(name string) ([]*Attestation, error) {
	if ac.Edges.namedAttestations == nil {
		return nil, &NotLoadedError{edge: name}
	}
	nodes, ok := ac.Edges.namedAttestations[name]
	if !ok {
		return nil, &NotLoadedError{edge: name}
	}
	return nodes, nil
}

func (ac *AttestationCollection) appendNamedAttestations(name string, edges ...*Attestation) {
	if ac.Edges.namedAttestations == nil {
		ac.Edges.namedAttestations = make(map[string][]*Attestation)
	}
	if len(edges) == 0 {
		ac.Edges.namedAttestations[name] = []*Attestation{}
	} else {
		ac.Edges.namedAttestations[name] = append(ac.Edges.namedAttestations[name], edges...)
	}
}

// AttestationCollections is a parsable slice of AttestationCollection.
type AttestationCollections []*AttestationCollection
