// Code generated by ent, DO NOT EDIT.

package ent

import (
	"context"
	"errors"
	"fmt"

	"entgo.io/ent/dialect/sql/sqlgraph"
	"entgo.io/ent/schema/field"
	"github.com/google/uuid"
	"github.com/in-toto/archivista/ent/dsse"
	"github.com/in-toto/archivista/ent/signature"
	"github.com/in-toto/archivista/ent/timestamp"
)

// SignatureCreate is the builder for creating a Signature entity.
type SignatureCreate struct {
	config
	mutation *SignatureMutation
	hooks    []Hook
}

// SetKeyID sets the "key_id" field.
func (sc *SignatureCreate) SetKeyID(s string) *SignatureCreate {
	sc.mutation.SetKeyID(s)
	return sc
}

// SetSignature sets the "signature" field.
func (sc *SignatureCreate) SetSignature(s string) *SignatureCreate {
	sc.mutation.SetSignature(s)
	return sc
}

// SetID sets the "id" field.
func (sc *SignatureCreate) SetID(u uuid.UUID) *SignatureCreate {
	sc.mutation.SetID(u)
	return sc
}

// SetNillableID sets the "id" field if the given value is not nil.
func (sc *SignatureCreate) SetNillableID(u *uuid.UUID) *SignatureCreate {
	if u != nil {
		sc.SetID(*u)
	}
	return sc
}

// SetDsseID sets the "dsse" edge to the Dsse entity by ID.
func (sc *SignatureCreate) SetDsseID(id uuid.UUID) *SignatureCreate {
	sc.mutation.SetDsseID(id)
	return sc
}

// SetNillableDsseID sets the "dsse" edge to the Dsse entity by ID if the given value is not nil.
func (sc *SignatureCreate) SetNillableDsseID(id *uuid.UUID) *SignatureCreate {
	if id != nil {
		sc = sc.SetDsseID(*id)
	}
	return sc
}

// SetDsse sets the "dsse" edge to the Dsse entity.
func (sc *SignatureCreate) SetDsse(d *Dsse) *SignatureCreate {
	return sc.SetDsseID(d.ID)
}

// AddTimestampIDs adds the "timestamps" edge to the Timestamp entity by IDs.
func (sc *SignatureCreate) AddTimestampIDs(ids ...uuid.UUID) *SignatureCreate {
	sc.mutation.AddTimestampIDs(ids...)
	return sc
}

// AddTimestamps adds the "timestamps" edges to the Timestamp entity.
func (sc *SignatureCreate) AddTimestamps(t ...*Timestamp) *SignatureCreate {
	ids := make([]uuid.UUID, len(t))
	for i := range t {
		ids[i] = t[i].ID
	}
	return sc.AddTimestampIDs(ids...)
}

// Mutation returns the SignatureMutation object of the builder.
func (sc *SignatureCreate) Mutation() *SignatureMutation {
	return sc.mutation
}

// Save creates the Signature in the database.
func (sc *SignatureCreate) Save(ctx context.Context) (*Signature, error) {
	sc.defaults()
	return withHooks(ctx, sc.sqlSave, sc.mutation, sc.hooks)
}

// SaveX calls Save and panics if Save returns an error.
func (sc *SignatureCreate) SaveX(ctx context.Context) *Signature {
	v, err := sc.Save(ctx)
	if err != nil {
		panic(err)
	}
	return v
}

// Exec executes the query.
func (sc *SignatureCreate) Exec(ctx context.Context) error {
	_, err := sc.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (sc *SignatureCreate) ExecX(ctx context.Context) {
	if err := sc.Exec(ctx); err != nil {
		panic(err)
	}
}

// defaults sets the default values of the builder before save.
func (sc *SignatureCreate) defaults() {
	if _, ok := sc.mutation.ID(); !ok {
		v := signature.DefaultID()
		sc.mutation.SetID(v)
	}
}

// check runs all checks and user-defined validators on the builder.
func (sc *SignatureCreate) check() error {
	if _, ok := sc.mutation.KeyID(); !ok {
		return &ValidationError{Name: "key_id", err: errors.New(`ent: missing required field "Signature.key_id"`)}
	}
	if v, ok := sc.mutation.KeyID(); ok {
		if err := signature.KeyIDValidator(v); err != nil {
			return &ValidationError{Name: "key_id", err: fmt.Errorf(`ent: validator failed for field "Signature.key_id": %w`, err)}
		}
	}
	if _, ok := sc.mutation.Signature(); !ok {
		return &ValidationError{Name: "signature", err: errors.New(`ent: missing required field "Signature.signature"`)}
	}
	if v, ok := sc.mutation.Signature(); ok {
		if err := signature.SignatureValidator(v); err != nil {
			return &ValidationError{Name: "signature", err: fmt.Errorf(`ent: validator failed for field "Signature.signature": %w`, err)}
		}
	}
	return nil
}

func (sc *SignatureCreate) sqlSave(ctx context.Context) (*Signature, error) {
	if err := sc.check(); err != nil {
		return nil, err
	}
	_node, _spec := sc.createSpec()
	if err := sqlgraph.CreateNode(ctx, sc.driver, _spec); err != nil {
		if sqlgraph.IsConstraintError(err) {
			err = &ConstraintError{msg: err.Error(), wrap: err}
		}
		return nil, err
	}
	if _spec.ID.Value != nil {
		if id, ok := _spec.ID.Value.(*uuid.UUID); ok {
			_node.ID = *id
		} else if err := _node.ID.Scan(_spec.ID.Value); err != nil {
			return nil, err
		}
	}
	sc.mutation.id = &_node.ID
	sc.mutation.done = true
	return _node, nil
}

func (sc *SignatureCreate) createSpec() (*Signature, *sqlgraph.CreateSpec) {
	var (
		_node = &Signature{config: sc.config}
		_spec = sqlgraph.NewCreateSpec(signature.Table, sqlgraph.NewFieldSpec(signature.FieldID, field.TypeUUID))
	)
	if id, ok := sc.mutation.ID(); ok {
		_node.ID = id
		_spec.ID.Value = &id
	}
	if value, ok := sc.mutation.KeyID(); ok {
		_spec.SetField(signature.FieldKeyID, field.TypeString, value)
		_node.KeyID = value
	}
	if value, ok := sc.mutation.Signature(); ok {
		_spec.SetField(signature.FieldSignature, field.TypeString, value)
		_node.Signature = value
	}
	if nodes := sc.mutation.DsseIDs(); len(nodes) > 0 {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.M2O,
			Inverse: true,
			Table:   signature.DsseTable,
			Columns: []string{signature.DsseColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: sqlgraph.NewFieldSpec(dsse.FieldID, field.TypeUUID),
			},
		}
		for _, k := range nodes {
			edge.Target.Nodes = append(edge.Target.Nodes, k)
		}
		_node.dsse_signatures = &nodes[0]
		_spec.Edges = append(_spec.Edges, edge)
	}
	if nodes := sc.mutation.TimestampsIDs(); len(nodes) > 0 {
		edge := &sqlgraph.EdgeSpec{
			Rel:     sqlgraph.O2M,
			Inverse: false,
			Table:   signature.TimestampsTable,
			Columns: []string{signature.TimestampsColumn},
			Bidi:    false,
			Target: &sqlgraph.EdgeTarget{
				IDSpec: sqlgraph.NewFieldSpec(timestamp.FieldID, field.TypeUUID),
			},
		}
		for _, k := range nodes {
			edge.Target.Nodes = append(edge.Target.Nodes, k)
		}
		_spec.Edges = append(_spec.Edges, edge)
	}
	return _node, _spec
}

// SignatureCreateBulk is the builder for creating many Signature entities in bulk.
type SignatureCreateBulk struct {
	config
	err      error
	builders []*SignatureCreate
}

// Save creates the Signature entities in the database.
func (scb *SignatureCreateBulk) Save(ctx context.Context) ([]*Signature, error) {
	if scb.err != nil {
		return nil, scb.err
	}
	specs := make([]*sqlgraph.CreateSpec, len(scb.builders))
	nodes := make([]*Signature, len(scb.builders))
	mutators := make([]Mutator, len(scb.builders))
	for i := range scb.builders {
		func(i int, root context.Context) {
			builder := scb.builders[i]
			builder.defaults()
			var mut Mutator = MutateFunc(func(ctx context.Context, m Mutation) (Value, error) {
				mutation, ok := m.(*SignatureMutation)
				if !ok {
					return nil, fmt.Errorf("unexpected mutation type %T", m)
				}
				if err := builder.check(); err != nil {
					return nil, err
				}
				builder.mutation = mutation
				var err error
				nodes[i], specs[i] = builder.createSpec()
				if i < len(mutators)-1 {
					_, err = mutators[i+1].Mutate(root, scb.builders[i+1].mutation)
				} else {
					spec := &sqlgraph.BatchCreateSpec{Nodes: specs}
					// Invoke the actual operation on the latest mutation in the chain.
					if err = sqlgraph.BatchCreate(ctx, scb.driver, spec); err != nil {
						if sqlgraph.IsConstraintError(err) {
							err = &ConstraintError{msg: err.Error(), wrap: err}
						}
					}
				}
				if err != nil {
					return nil, err
				}
				mutation.id = &nodes[i].ID
				mutation.done = true
				return nodes[i], nil
			})
			for i := len(builder.hooks) - 1; i >= 0; i-- {
				mut = builder.hooks[i](mut)
			}
			mutators[i] = mut
		}(i, ctx)
	}
	if len(mutators) > 0 {
		if _, err := mutators[0].Mutate(ctx, scb.builders[0].mutation); err != nil {
			return nil, err
		}
	}
	return nodes, nil
}

// SaveX is like Save, but panics if an error occurs.
func (scb *SignatureCreateBulk) SaveX(ctx context.Context) []*Signature {
	v, err := scb.Save(ctx)
	if err != nil {
		panic(err)
	}
	return v
}

// Exec executes the query.
func (scb *SignatureCreateBulk) Exec(ctx context.Context) error {
	_, err := scb.Save(ctx)
	return err
}

// ExecX is like Exec, but panics if an error occurs.
func (scb *SignatureCreateBulk) ExecX(ctx context.Context) {
	if err := scb.Exec(ctx); err != nil {
		panic(err)
	}
}
